Package: hydroloom
Title: Utilities to Weave Hydrologic Fabrics
Version: 1.2.0
Authors@R: c(
    person(given = "David", 
           family = "Blodgett", 
           role = c("aut", "cre"), 
           email = "dblodgett@usgs.gov",
           comment = c(ORCID = "0000-0001-9489-1710")),
    person(given = "Andrew",
           family = "Psoras",
           role = "ctb",
           email = "apsoras@usgs.gov"))
Description: A collection of utilities that support creation of network attributes for hydrologic networks. Methods and algorithms implemented are documented in Moore et al. (2019) <doi:10.3133/ofr20191096>), Cormen and Leiserson (2022) <ISBN:9780262046305> and Verdin and Verdin (1999) <doi:10.1016/S0022-1694(99)00011-6>.  
Depends: R (>= 4.1.0)
Imports: dplyr, data.table, sf, units, stats, methods, utils, pbapply,
        tidyr, RANN, rlang, fastmap
Suggests: testthat, nhdplusTools, future, lwgeom, future.apply, knitr,
        gifski, mapview, webshot, geos
License: CC0
Encoding: UTF-8
RoxygenNote: 7.3.3
Config/testthat/edition: 3
VignetteBuilder: knitr
Language: en-US
URL: https://github.com/DOI-USGS/hydroloom,
        https://doi-usgs.github.io/hydroloom/
NeedsCompilation: no
Packaged: 2026-05-21 19:47:05 UTC; dblodgett
Author: David Blodgett [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9489-1710>),
  Andrew Psoras [ctb]
Maintainer: David Blodgett <dblodgett@usgs.gov>
Repository: CRAN
Date/Publication: 2026-05-23 11:20:12 UTC
Built: R 4.6.0; ; 2026-05-23 15:12:24 UTC; unix
