# Generated by roxygen2: do not edit by hand

S3method(accumulate_downstream,data.frame)
S3method(accumulate_downstream,hy)
S3method(accumulate_downstream,hy_flownetwork)
S3method(accumulate_downstream,hy_node)
S3method(accumulate_downstream,hy_topo)
S3method(add_divergence,data.frame)
S3method(add_divergence,hy)
S3method(add_divergence,hy_node)
S3method(add_divergence,hy_topo)
S3method(add_levelpaths,data.frame)
S3method(add_levelpaths,hy)
S3method(add_levelpaths,hy_flownetwork)
S3method(add_levelpaths,hy_node)
S3method(add_levelpaths,hy_topo)
S3method(add_measures,data.frame)
S3method(add_measures,hy)
S3method(add_pathlength,data.frame)
S3method(add_pathlength,hy)
S3method(add_pathlength,hy_node)
S3method(add_pathlength,hy_topo)
S3method(add_pfafstetter,data.frame)
S3method(add_pfafstetter,hy)
S3method(add_pfafstetter,hy_leveled)
S3method(add_pfafstetter,hy_node)
S3method(add_pfafstetter,hy_topo)
S3method(add_return_divergence,data.frame)
S3method(add_return_divergence,hy)
S3method(add_return_divergence,hy_node)
S3method(add_return_divergence,hy_topo)
S3method(add_streamlevel,data.frame)
S3method(add_streamlevel,hy)
S3method(add_streamlevel,hy_leveled)
S3method(add_streamlevel,hy_topo)
S3method(add_streamorder,data.frame)
S3method(add_streamorder,hy)
S3method(add_streamorder,hy_flownetwork)
S3method(add_streamorder,hy_node)
S3method(add_streamorder,hy_topo)
S3method(add_toids,data.frame)
S3method(add_toids,hy)
S3method(add_toids,hy_node)
S3method(add_toids,hy_topo)
S3method(add_topo_sort,data.frame)
S3method(add_topo_sort,hy)
S3method(add_topo_sort,hy_flownetwork)
S3method(add_topo_sort,hy_node)
S3method(add_topo_sort,hy_topo)
S3method(check_valid,default)
S3method(check_valid,sf)
S3method(check_valid,sfc)
S3method(dissolve_polygons,default)
S3method(dissolve_polygons,sf)
S3method(dissolve_polygons,sfc)
S3method(get_bridge_flowlines,data.frame)
S3method(get_bridge_flowlines,hy)
S3method(get_bridge_flowlines,hy_flownetwork)
S3method(get_bridge_flowlines,hy_node)
S3method(get_bridge_flowlines,hy_topo)
S3method(index_points_to_lines,data.frame)
S3method(index_points_to_lines,hy)
S3method(make_attribute_topology,data.frame)
S3method(make_attribute_topology,hy)
S3method(make_index_ids,data.frame)
S3method(make_index_ids,hy)
S3method(make_index_ids,hy_flownetwork)
S3method(make_index_ids,hy_topo)
S3method(make_node_topology,data.frame)
S3method(make_node_topology,hy)
S3method(make_node_topology,hy_flownetwork)
S3method(make_node_topology,hy_topo)
S3method(navigate_hydro_network,data.frame)
S3method(navigate_hydro_network,hy)
S3method(navigate_hydro_network,hy_leveled)
S3method(navigate_hydro_network,hy_node)
S3method(navigate_hydro_network,hy_topo)
S3method(print,hy_flownetwork)
S3method(print,hy_node)
S3method(print,hy_topo)
S3method(print,hydroloom_names)
S3method(sort_network,data.frame)
S3method(sort_network,hy)
S3method(sort_network,hy_flownetwork)
S3method(sort_network,hy_node)
S3method(sort_network,hy_topo)
S3method(subset_network,data.frame)
S3method(subset_network,hy)
S3method(subset_network,hy_node)
S3method(subset_network,hy_topo)
S3method(to_flownetwork,data.frame)
S3method(to_flownetwork,hy)
S3method(to_flownetwork,hy_leveled)
S3method(to_flownetwork,hy_node)
S3method(to_flownetwork,hy_topo)
export(accumulate_downstream)
export(add_divergence)
export(add_levelpaths)
export(add_measures)
export(add_pathlength)
export(add_pfafstetter)
export(add_return_divergence)
export(add_streamlevel)
export(add_streamorder)
export(add_toids)
export(add_topo_sort)
export(align_names)
export(check_hy_graph)
export(check_valid)
export(disambiguate_indexes)
export(dissolve_polygons)
export(fix_flowdir)
export(format_index_ids)
export(get_bridge_flowlines)
export(get_hydro_location)
export(get_node)
export(get_partial_length)
export(hy)
export(hy_capabilities)
export(hy_network_type)
export(hy_reverse)
export(hydroloom_name_definitions)
export(hydroloom_names)
export(index_points_to_lines)
export(index_points_to_waterbodies)
export(is.hy)
export(is_dendritic)
export(make_attribute_topology)
export(make_fromids)
export(make_index_ids)
export(make_node_topology)
export(navigate_connected_paths)
export(navigate_hydro_network)
export(navigate_network_dfs)
export(rename_geometry)
export(rescale_measures)
export(sort_network)
export(st_compatibalize)
export(subset_network)
export(to_flownetwork)
importFrom(RANN,nn2)
importFrom(data.table,.N)
importFrom(data.table,.SD)
importFrom(data.table,as.data.table)
importFrom(data.table,copy)
importFrom(data.table,data.table)
importFrom(data.table,fcase)
importFrom(data.table,rbindlist)
importFrom(data.table,setcolorder)
importFrom(data.table,setnames)
importFrom(data.table,setorder)
importFrom(data.table,setorderv)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,as_tibble)
importFrom(dplyr,between)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,group_split)
importFrom(dplyr,lag)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,right_join)
importFrom(dplyr,row_number)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,tibble)
importFrom(dplyr,ungroup)
importFrom(fastmap,fastqueue)
importFrom(fastmap,faststack)
importFrom(methods,as)
importFrom(pbapply,pbapply)
importFrom(pbapply,pblapply)
importFrom(pbapply,pboptions)
importFrom(pbapply,pbsapply)
importFrom(rlang,":=")
importFrom(sf,"st_geometry<-")
importFrom(sf,st_as_sf)
importFrom(sf,st_buffer)
importFrom(sf,st_cast)
importFrom(sf,st_coordinates)
importFrom(sf,st_crs)
importFrom(sf,st_drop_geometry)
importFrom(sf,st_geometry)
importFrom(sf,st_geometry_type)
importFrom(sf,st_intersects)
importFrom(sf,st_is_longlat)
importFrom(sf,st_join)
importFrom(sf,st_linestring)
importFrom(sf,st_point)
importFrom(sf,st_reverse)
importFrom(sf,st_segmentize)
importFrom(sf,st_sf)
importFrom(sf,st_sfc)
importFrom(sf,st_transform)
importFrom(sf,st_zm)
importFrom(stats,na.omit)
importFrom(stats,setNames)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,replace_na)
importFrom(tidyr,unnest)
importFrom(units,as_units)
importFrom(units,set_units)
importFrom(utils,adist)
importFrom(utils,combn)
importFrom(utils,setTxtProgressBar)
importFrom(utils,tail)
importFrom(utils,txtProgressBar)
